header {
	package edu.usfca.syndiag;
	import java.awt.*;
	import java.awt.event.*;
	import javax.swing.*;
}

/** A tree grammar set dimension of each node in the Grammar AST
 *
 *  Jia Zheng
 *  University of San Francisco
 *  Oct. 1, 2003
 */


class SetDimension extends TreeParser;

options {
	importVocab = ANTLR;
    codeGenBitsetTestThreshold=999;
}

{
public static final int ELEMENT_FONT_SIZE = 15;
public static final int RULE_DEF_FONT_SIZE = 20;

public static int getStringWidth(String str, Font f){
	JFrame jframe = new JFrame();
	jframe.pack(); 
	Graphics g = jframe.getGraphics(); 
	g.setFont(f);
	FontMetrics fm = g.getFontMetrics();
	int w = fm.stringWidth(str);
	return w;
}

}
grammar
	{GrammarAST g = (GrammarAST)#grammar;
	 int w = 0;
	 int h = 0;
	 Dimension c;
	 }
    :   c=classDef // ignore 2nd (lexer) grammar for now
    	{((GrammarAST)g).setDim((int)c.getWidth(), (int)c.getHeight());
    	 System.out.println("width = " + (int)c.getWidth() + ", height = " + (int)c.getHeight());
    	}
    ;

classDef returns [Dimension d]
	{GrammarAST c = (GrammarAST)#classDef;
	 int w = 0;
	 int h = 0;
	 d = new Dimension(w, h);
	 Dimension d1, d2, d3;}
    :   #( PARSER d1=rules )
    		{w = (int)d1.getWidth();
    		 h = (int)d1.getHeight();
    		 d = new Dimension(w+200, h+200);
    		 }
    |   #( TREE_PARSER d2=rules )
    		{w = (int)d2.getWidth();
    		 h = (int)d2.getHeight();
    		 d = new Dimension(w+200, h+200);    		 
    		 }    
    |   #( LEXER d3=rules )
    		{w = (int)d3.getWidth();
    		 h = (int)d3.getHeight();
    		 d = new Dimension(w+200, h+200);    		 
    		 }   
		
    ;

rules returns [Dimension d]
	{int w = 0;
	 int h = 0;
	 int deltaY = 0;
	 d = new Dimension(w, h);
	}
    :      (r : rule
    			{w = Math.max(w, ((GrammarAST)r).getWidth());
          		 h += ((GrammarAST)r).getHeight() + deltaY;
          		 deltaY = 60;
          		}
    		)+
    		{d = new Dimension(w, h);}
    ;

rule
	{GrammarAST rl = (GrammarAST)#rule;
	 int w = 0; 
	 int h = 0; 
	 Dimension d;
	 }
    :   #( RULE r:RULE_REF
    			{w = getStringWidth(r.getText(), new Font("Monospaces", Font.BOLD, RULE_DEF_FONT_SIZE)) +  100;
    			 ((GrammarAST)r).setDim(w, RULE_DEF_FONT_SIZE);
    			}
           #(BLOCK d=b:block)
           		{w += (int)d.getWidth(); 
           		 h += (int)d.getHeight();
           		}
           EOR
         )
         {((GrammarAST)rl).setDim(w, h);}
    ;

block returns [Dimension d]
	{GrammarAST b = (GrammarAST)#block;
	 int w = 0; 
	 int h = 0; 
	 int deltaY = 0;
	 d = new Dimension(0,0);
	}
    :   ( a : alternative
         	{w = Math.max(w, ((GrammarAST)a).getWidth());
          	 h += ((GrammarAST)a).getHeight() + deltaY;
          	 deltaY = 30;
          	}
        )+        
        {((GrammarAST)b).setBlockDim(w, h);
         d = new Dimension(w, h);
        }
    ;

alternative
	{int w = 0;
	 int h = 0; 
	 int deltaX = 0;
	}
    :   #( a : ALT
    		(e : element
         		{w += ((GrammarAST)e).getWidth() + deltaX;
          		 h = Math.max(h, ((GrammarAST)e).getHeight());
          		 deltaX = 50;
         		}
            )+
         )
         {((GrammarAST)a).setDim(w, h);}
    ;

element
	{GrammarAST e = (GrammarAST)#element;
	 Dimension b;
	 int w; 
	 int h;
	}
    :   a:atom
    		{e.setDim(((GrammarAST)a).getWidth(), ((GrammarAST)a).getHeight());}
    |   #(NOT elt:element)
    	 	{e.setDim(((GrammarAST)elt).getWidth(), ((GrammarAST)elt).getHeight());}
    |   #(tr:TOKEN_RANGE a1:atom a2:atom)
    		{w = ((GrammarAST)a1).getWidth() + ((GrammarAST)a2).getWidth();
    		 h = Math.max(((GrammarAST)a1).getHeight(), ((GrammarAST)a2).getHeight());
    		 e.setDim(w, h);
    		}
    |   #(CHAR_RANGE c1:CHAR_LITERAL c2:CHAR_LITERAL)
    		{w = ((GrammarAST)c1).getWidth() + ((GrammarAST)c2).getWidth();
    		 h = Math.max(((GrammarAST)c1).getHeight(), ((GrammarAST)c2).getHeight());
    	  	 e.setDim(w, h);
    	  	}    
    |   eb:ebnf
    	 	{e.setDim(((GrammarAST)eb).getWidth(), ((GrammarAST)eb).getHeight());}
    |   t:tree
    	 	{e.setDim(((GrammarAST)t).getWidth(), ((GrammarAST)t).getHeight());}
    |   #(SYNPRED b=block)
	  		{e.setDim((int)b.getWidth(), (int)b.getHeight());}
    |   ACTION
    |   SEMPRED
    |   EPSILON
    ;

ebnf
	{Dimension b;}
	:   #(bb:BLOCK b=block)
			{((GrammarAST)bb).setDim((int)b.getWidth() + 30, (int)b.getHeight());}
    |   #(o:OPTIONAL b=block )
    		{((GrammarAST)o).setDim((int)b.getWidth() + 30, (int)b.getHeight());}
    |   #(c:CLOSURE b=block)
    		{((GrammarAST)c).setDim((int)b.getWidth()+30, (int)b.getHeight());}
    |   #(p:POSITIVE_CLOSURE b=block )
    		{((GrammarAST)p).setDim((int)b.getWidth()+30, (int)b.getHeight());}
    ;

tree
	:   #( t:TREE_BEGIN a:atom
							{int w = ((GrammarAST)a).getWidth();
							 int h = ((GrammarAST)a).getHeight();
							}
						 (e:element
						 	{w += ((GrammarAST)e).getWidth() + ((GrammarAST)e).deltaX;
						 	 h = Math.max(h, ((GrammarAST)e).getHeight());
						 	}
						 )*
		)
		{((GrammarAST)t).setDim(w, h);}
    ;

atom
	{int w = 0; 
	 int h = 20;
	}
	:   r:RULE_REF
			{w = getStringWidth(r.getText(), new Font("Monospaces", Font.PLAIN, ELEMENT_FONT_SIZE)) + 30;
		  	 ((GrammarAST)r).setDim(w, h);
		 	}
    |   t:TOKEN_REF
		 	{w = getStringWidth(t.getText(), new Font("Monospaces", Font.PLAIN, ELEMENT_FONT_SIZE)) + 30;
		 	 ((GrammarAST)t).setDim(w, h);
		 	}
    |   c:CHAR_LITERAL
		 	{w = getStringWidth(c.getText(), new Font("Monospaces", Font.PLAIN, ELEMENT_FONT_SIZE)) + 30;
		 	 ((GrammarAST)c).setDim(w, h);
		 	}    
    |   s:STRING_LITERAL
		 	{w = getStringWidth(s.getText(), new Font("Monospaces", Font.PLAIN, ELEMENT_FONT_SIZE)) + 30;
		 	 ((GrammarAST)s).setDim(w, h);
		 	}    
    |   wi:WILD_CARD
		 	{w = getStringWidth(wi.getText(), new Font("Monospaces", Font.PLAIN, ELEMENT_FONT_SIZE)) + 30;
		 	 ((GrammarAST)wi).setDim(w, h);
		 	}    
    ;
